/* 	
 * Title        transam2.c     
 * Programmer	Jason Grzywna
 * Date         March 23, 2000
 * Version		2.0c			               										        
 * Description									
 *    A very simple collision avoidance program. TJ PRO will read 	
 *    each IR detectors, and turn away from any obstacles in its	
 *    path. Also, if something hits TJ PRO's bumper, it will back up,	
 *    turn, and go on.							
 *			                                               
 */

#include <tjpbase.h>
#include <stdio.h>


#define START 		while(BUMPER<120)
#define LIGHTS_PORT	*(unsigned char *)(0x6000)
#define DebugSW		analog(7)
#define CDSnose		analog(1)
#define CDS_L		analog(5)
#define CDS_R		analog(6)

int poweronselftest(void);
void avoidance(void);
int IR_processor();
int Velocity_processor();
int Accelerator(int speed);
void WhiteRabbit();
void turn(int);
void light(int);



	int steering_center = 2500; /*THESE SHOULD BE GLOBAL*/
	int far_left = 1650;
	int far_right = 3800;
	int current_speed = 0;
	int spd_factorL, spd_factorR;
	int irL, irR, IRprocessed, d_of_turn, xx, yy, i;


void main(void)
{

	int temp1 = 99;
		
	init_analog();
	init_motortjp();
	init_clocktjp();
	init_servotjp();
	
	SET_BIT(LIGHTS_PORT, 0xFF);  /* TURN OFF LIGHTS INITIALLY */
	
 	if (DebugSW > 100)
 	{
 		temp1 = poweronselftest();
 		printf("\nerrors = %d\n", temp1);
 		/*WhiteRabbit();*/
 	}
 	
 	
 	START;
 	IRE_ON;
 	while(1)
 	{
 	IRprocessed = IR_processor();
 	 	
 	printf("\n%d\n", IRprocessed);
 	wait(400);
 	Velocity_processor();
 	avoidance();
  	}
  
       
}


int poweronselftest(void)
{
	int errors = 0;
	int lir, rir;
	
	/* STEERING TEST */

	servo(0, steering_center);
	printf("\ncenter\n");
	wait(300);
	servo(0, far_left);
	printf("\nleft\n");
	wait(300);
	servo(0, far_right);
	printf("\nright\n");
	wait(300);
	servo(0, steering_center);
	wait(300);
	
	/* IR TEST */
	 
	IRE_ON;
	lir = LEFT_IR;
	rir = RIGHT_IR;
	
	if ((lir>85)&&(lir<135))
	{}
	else 
		errors = errors + 1;
	
	if ((rir>85)&&(rir<135))
	{}
	else
		errors = errors + 1;
	IRE_OFF;
		
	/* BUMP SWITCH TEST */
	
	if (analog(0) > 3)
		errors = errors + 1;
	
	/* MOTOR TEST */
	
	motorp(0, 100);
	motorp(1, 100);
	wait(100);
	motorp(0, -100);
	motorp(1, -100);
	wait(100);
	motorp(0, 0);
	motorp(1, 0);
	
return errors;	
}




void avoidance(void)
{
	
	
	
	
	
	
	
	printf("turn(");
	xx = read_int();
	printf("speed(");
	yy = read_int();
	
	turn(xx);
	Accelerator(yy);	 
		
/*	for(i=-3; i<4; i+=1) 			
        {
          printf("\n\n%d\n\n",i);
          turn(i);  
          wait(40);     
        }
     for(i=3; i>-4; i-=1) 
        {
          printf("\n\n%d\n\n",i);
          turn(i);
          wait(40);
        }       
*/
}


	
int IR_processor(void)
{
	int IRout, IRout1, IRtemp, irLtemp, irRtemp;
		
	irL = analog(3);
	irR = analog(2);
	
	
	printf("\nIR Left - %d", irL);
	printf("\nIR Right - %d", irR);
		irLtemp = -irL;
		irRtemp = irR;
		IRtemp = (irLtemp + irRtemp);

	
	/*IRout1 = (IRtemp * 2.5);
	printf("\nIRout1 - %d", IRout1);*/ 				/* Scales to -100 -> 100 */
		
	if ((IRtemp < 12) && (IRtemp > -12)) /* || (IRtemp = 0))		/* Processes Depth is object is centered */		
	{ 
		if ((irL < 95) && (irR < 95))
		{
			IRout = 11;
		}
		else if ((irL < 105) && (irR < 105))
		{
			IRout = 22;
		}
		else if ((irL < 115) && (irR < 115))
		{
			IRout = 33;
		}
		else if ((irL < 124) && (irR < 124))
		{
			IRout = 44;
		}
		else if ((irL > 125) && (irR > 125))
		{
			IRout = 55;
			printf("CLOSE");
		}
	}
	else
	{
		IRout = IRtemp; /*(IRout1 / 10);*/
		printf("\n\nIRout!!! - %d", IRout);
	}
		
	
	return IRout;
}



int Velocity_processor()
{
}

int Accelerator(int speed)
{
	int L_Speed, R_Speed;
	 
	
	
	if (speed <= 0)
	{
		if (speed <= -5)
		{
			L_Speed = -99;
			R_Speed = 4;
		}
		else if (speed <= -4)
		{
			L_Speed = -90;
			R_Speed = 14;
		}
		else if (speed <= -3)	
		{
			L_Speed = -80;
			R_Speed = 24;
			//perfect
		}
		else if (speed <= -2)
		{
			L_Speed = -70;
			R_Speed = 34;
			//near perfect
		}
		else if (speed <= -1)
		{
			L_Speed = -62;
			R_Speed = 44;
		}
		else if (speed <= 0)
		{
			L_Speed = 0;
			R_Speed = 0;
		}
	}
	else if (speed > 0)
	{
		if (speed >= 5)
		{
			L_Speed = -6;
			R_Speed = 99;
		}
		else if (speed >= 4)
		{
			L_Speed = -11;
			R_Speed = 94;
		}
		else if (speed >= 3)
		{
			L_Speed = -19;
			R_Speed = 84;
		}
		else if (speed >= 2)
		{
			L_Speed = -29;
			R_Speed = 74;
		}
		else if (speed >= 1)
		{
			L_Speed = -41;
			R_Speed = 64;
		}
		
	}
	else
	{
			L_Speed = 0;
			R_Speed = 0;
	}		

	
	
	if (speed < current_speed)
	{
		SET_BIT(LIGHTS_PORT, 0x04);
	}
	else 
		CLEAR_BIT(LIGHTS_PORT, 0x04);
		
	motorp(0, L_Speed);
	motorp(1, R_Speed);

	return current_speed;
}

void turn(int dot)
{
	int degree;
	
	if (dot < 0)
	{
		if (dot <= -3)
		{
			degree = 1650;
			CLEAR_BIT(LIGHTS_PORT, 0x01);
		}
		else if (dot <= -2)
		{
			degree = 2000;
			CLEAR_BIT(LIGHTS_PORT, 0x01);
		}
		else if (dot <= -1)
		{
			degree = 2300;
			CLEAR_BIT(LIGHTS_PORT, 0x01);
		}
	}
	else if (dot > 0)
	{
		if (dot >= 3)
		{
			degree = 3800;
			CLEAR_BIT(LIGHTS_PORT, 0x02);
		}
		else if (dot >= 2)
		{
			degree = 2900;
			CLEAR_BIT(LIGHTS_PORT, 0x02);
		}
		else if (dot >= 1)
		{
			degree = 2700;
			CLEAR_BIT(LIGHTS_PORT, 0x02);
		}
	}
	else
	{
		degree = 2500;
		SET_BIT(LIGHTS_PORT, 0x03);
	}
	
	printf("%d", degree);
	servo(0, degree);	

}

void light(int bulb)
{





}

void WhiteRabbit()
{
char c1, clear[]= "\x1b\x5B\x32\x4A\x04";
char place[]= "\x1b[1;1H";	/*Home*/	
char c2;

printf("%s", clear);  
printf("%s", place);

printf("\tTitle\t\tWhite Rabbit\n"
  "\tProgrammer\tJason Grzywna\n"
  "\tDate\t\tMarch, 2000\n"
  "\tVersion\t\t1.0\n\n");
  
printf("TRANS AM Manual Driver\n\n");
printf("Enter Command: W=forward, X=reverse, D=increase R, A=increase L\n");

c2 = get_char();
}
  
  
  
