//transam.c
//**Final Version 2.3+
//
//WORKS PERFECT +-5% error
//**Start TransAm on White Printer Paper (3 layers)
//**Aquires track at virtually any angle.
//**Changes lane when obstacle is struck
//**Changes lane when object is seen in path
//**Randomness 
//**Better lane processing
//
//
//				Reverses and re-angles
//					  \ ^^ /
//	Successful Aquire  \  /	Successful Aquire
//        				\/	(in this angle ~0 to 75 deg
//			___________________________
//
// Jason W. Grzywna  4-21-2000  v2.3+ TransAm



#include <transam.h>
#include <stdio.h>

#define Nose_IR analog(4)

void autocalibrate();			
void off_track();
void aquire_track(int);			// 0=aquire_R 1=aquire_L 9=reposition
void on_track();				
void turn(int);					// -3=full_L  0=center  3=full_R
int Accelerator(int);			// -5=full_fwd  0=stop  5=full_rev
int IR_procT();					//IR Processor with Turning output	
int CDS_procT();				//CDS Processor with Turning output
void turn_toward_ir();		
void Random_backup();	


int CL_calib, CR_calib, CN_calib;
int velocity, i, j, k, x, nosej;
int aquire_var;					// 0=right 1=left
int current_speed;
int on_track_var;
int last_seen;
int current_lane = 0;
int traveling_R;
int lane_set = 0;
int xx, jj, kk, ii;

void main()
{
	init_analog();				//Setup TJP board
	init_motortjp();
	init_clocktjp();
	init_servotjp();
	
	NoSignal;					//No Lights on
	START;						//Wait for rear bumper press
	IRE_ON;						//Illuminate front IRs
	autocalibrate();
	wait(200);
	on_track_var = 0;
	
	
	while(1)
	{
	if (on_track_var > 0)
	{
		on_track();				//Track Follow
	}
	else
		off_track();			//Avoidance+Searching
	}
	
	
}

void on_track()
{
	int temptrack, z, noseO, left_cdsO, right_cdsO;
	
	noseO 		= analog(1);
	left_cdsO 	= analog(5);
	right_cdsO	= analog(6);
	
	temptrack = CDS_procT();     //this routine sets last_seen var
	turn(temptrack);
	velocity = Accelerator(-3);
	
	if ((FRONT_BUMP) || (Nose_IR > 120))
	{
		
		
		if (FRONT_BUMP)
		{
			Accelerator(0);
			turn(0);					//turning angle should stay same to back along same turn
			wait(50);
			Accelerator(3);
			wait(800);
		}
		
		if ((traveling_R == 0) && (current_lane == 1))
		{
			turn(3);
			wait(800);
			jj = 0;
			while ((analog(6) < CR_calib) && (jj < 10))	//locks in an infinite loop
			{											//if turning out of wrong lane
				turn(3);
				Accelerator(-3);
				jj++;
				if (FRONT_BUMP)					//Back away from collision
				{
					Accelerator(4);
					turn_toward_ir();
					wait(800);
				}
			}
		
						
			on_track_var = 0;
			aquire_track(1);
		}
		else if ((traveling_R == 1) && (current_lane == 1))
		{
			turn(-3);
			wait(800);
			kk = 0;
			while ((analog(5) < CL_calib) && (kk < 10))
			{
				turn(-3);
				Accelerator(-3);
				kk++;
				if (FRONT_BUMP)					//Back away from collision
				{
					Accelerator(4);
					turn_toward_ir();
					wait(800);
				}
			}
			
			on_track_var = 0;
			aquire_track(0);
		}
		else if ((traveling_R == 0) && (current_lane == 2))
		{
			turn(-3);
			wait(800);
			xx = 0;
			while ((analog(5) < CL_calib) && (xx < 10))
			{
				turn(-3);
				Accelerator(-3);
				xx++;
				if (FRONT_BUMP)					//Back away from collision
				{
					Accelerator(4);
					turn_toward_ir();
					wait(800);
				}
			}
			
			current_lane = 0;
			on_track_var = 0;
			aquire_track(0);
		}
		else if ((traveling_R == 1) && (current_lane == 2))
		{
			turn(3);
			wait(800);
			ii = 0;
			while ((analog(6) < CR_calib) && (ii < 10))
			{
				turn(3);
				Accelerator(-3);
				ii++;
				if (FRONT_BUMP)					//Back away from collision
				{
					Accelerator(4);
					turn_toward_ir();
					wait(800);
				}
			}
			
			current_lane = 0;
			on_track_var = 0;
			aquire_track(1);
		}
	}
	
									//If lose track, resume Off_track()	
	 
	
	if ((noseO <= CN_calib) && (left_cdsO <= CL_calib) && (right_cdsO <= CR_calib))
	{
		z++;
	}
	else if ((noseO >= CN_calib) || (left_cdsO >= CL_calib) || (right_cdsO >= CR_calib))
	{
		z = 0;
	}
	
	if ((z > 0) && (z < 7))
	{
		x = 0;
		while ((noseO < CN_calib) && (x < 10))
		{
		if (FRONT_BUMP)					//Back away from collision
			{
			if (Nose_IR > 125)
			{
				Accelerator(4);
				Random_backup();
				goto JUMP2;
			}
				Accelerator(4);
				turn_toward_ir();
				wait(800);
			}
JUMP2:
		noseO = analog(1);
		turn(last_seen);
		x++;
		}
	}
	else if (z >= 7)
	{
		if ((last_seen == 3) && (traveling_R == 1))
		{
			 current_lane = 1;
		}
		else if ((last_seen == 3) && (traveling_R == 0))
		{
			current_lane = 0;
		}
		else if ((last_seen == -3) && (traveling_R == 1))
		{
			current_lane = 0;
		}
		else if ((last_seen == -3) && (traveling_R == 0))
		{
			current_lane = 1;
		}
		on_track_var = 0;
		off_track();
	}

}	

void off_track()
{

	int turning_IR, nose, left_cds, right_cds;
	
	turning_IR = IR_procT();		//Init IR Processor
	turn(turning_IR);				//Turn based on IR Proc
	velocity = Accelerator(-3);		//Set Accel to 3 out of 5	
	
	nose 		= analog(1);
	left_cds 	= analog(5);
	right_cds	= analog(6);
	
	if (FRONT_BUMP)					//Back away from collision
	{
		if (Nose_IR > 125)
		{
			Accelerator(4);
			Random_backup();
			goto JUMP;
		}
			
		Accelerator(4);
		turn_toward_ir();
		wait(800);
	}
JUMP:
	
	if (nose >= CN_calib)			//Initiate Track Aquire
	{
		
		velocity = Accelerator(-2);
		wait(100);					//EXPERIMENTAL
		
		left_cds = analog(5);
		right_cds = analog(6);
		
		if ((right_cds > CR_calib) && (left_cds > CL_calib))
		{
		aquire_track(9);				//Re-angle approach
		}
		else if (left_cds > CL_calib)
		{
		aquire_track(0);				//Aquire right
		}
		else if (right_cds > CR_calib)
		{
		aquire_track(1);				//Aquire left
		}
		
	}
	
}


void autocalibrate()
{

int cdsleft_t, cdsright_t, cdsnose_t;

									//Set THRESHOLDS

cdsleft_t 	= analog(5);
cdsright_t 	= analog(6);
cdsnose_t	= analog(1);

CL_calib = (cdsleft_t * 2.0);		//2.1		//Calculated Thresholds
CR_calib = (cdsright_t * 2.0);		//2.1  		//20% higher then the 
CN_calib = (cdsnose_t * 1.35);		//1.45		//inital white floor readings

/*
printf("\n\nleft calibrated: %d", CL_calib);
printf("\nright calibrated: %d", CR_calib);
printf("\nnose calibrated: %d", CN_calib);

while(1)
{
	printf("\tnose=%d", analog(1));
	printf("\tright=%d", analog(6));
	printf("\tleft=%d", analog(5));
	wait(50);

}
*/
 
}

void aquire_track(int aquire_var)
{
	if (aquire_var == 0)
	{
		turn(2);
		Accelerator(-1);
		wait(1250);
		nosej = analog(1);
		j = 0;
		
		while ((nosej < CN_calib) && (j < 150))
		{
			if (FRONT_BUMP)					//Back away from collision
			{
				Accelerator(4);
				turn_toward_ir();
				wait(800);
			}
			Accelerator(-1);
			turn(3);
			nosej = analog(1);
			j++;	
		}
		if (nosej > CN_calib)
		{
			turn(0);
			on_track_var = 1;
			if (lane_set == 0)
			{
				traveling_R = 1;
				lane_set = 1;
			}
			current_lane = current_lane + 1;
			on_track();	
		}
	}
	else if (aquire_var == 1)
	{
		turn(-2);
		Accelerator(-1);
		wait(1250);
		nosej = analog(1);
		k = 0;
		
		while ((nosej < CN_calib) && (k < 150))
		{
		
			if (FRONT_BUMP)					//Back away from collision
			{
				Accelerator(4);
				turn_toward_ir();
				wait(800);
			}
			Accelerator(-1);
			turn(-3);
			nosej = analog(1);
			k++;
		}
		if (nosej > CN_calib)
		{
			turn(0);
			on_track_var = 1;
			if (lane_set == 0)
			{
				traveling_R = 0;
				lane_set = 1;
			}
			current_lane = current_lane + 1;
			on_track();
		}
	}
	else if (aquire_var == 9)
	{
									//Re-angle script
		for (i = 0; i < 15; i++)	//wait(2000)
		{ 
		turn(-3);
		Accelerator(3);
		
			if (BACK_BUMP)
			{
				goto HERE;
			}
		}
		
		for (i = 0; i < 12; i++)	//wait(1500);
		{
		turn(0);
		Accelerator(3);
			
			if (BACK_BUMP)
			{
				goto HERE;
			}
		}
		
		for (i = 0; i < 1; i++)	//wait(50)
		{
		Accelerator(0);
		}
		
HERE:
		
		for (i = 0; i < 4; i++)	//wait(900)
		{
		turn(-1);
		Accelerator(-3);
		}
				
		off_track();
	}
	
}

void turn(int dot)
{
	int degree;
	
	if (dot < 0)
	{
		if (dot <= -3)
		{
			degree = 1500;
			LeftSignal;
			
		}
		else if (dot <= -2)
		{
			degree = 1834;
			LeftSignal;
			
		}
		else if (dot <= -1)
		{
			degree = 2167;
			LeftSignal;
			
		}
	}
	else if (dot > 0)
	{
		if (dot >= 3)
		{
			degree = 4000;
			RightSignal;
			
		}
		else if (dot >= 2)
		{
			degree = 3500;
			RightSignal;
			
		}
		else if (dot >= 1)
		{
			degree = 3000;
			RightSignal;
			
		}
	}
	else
	{
		degree = 2500;
		NoSignal;
	}
	
	servo(0, degree);
	wait(100);	

}	

int IR_procT()
{

int irT_L, irT_R, t_diff;
int tL = 0;
int tR = 0;
irT_L = analog(3);
irT_R = analog(2);

if (irT_L > 95)
{
	if (irT_L > 95)
	{
		tL = 1;
		if (irT_L > 105)
		{
			tL = tL + 1;	
			if (irT_L > 115)
			{
				tL = tL + 1;
			}
		}
	}
}
else
	{
		tL = 0;
	}	


if (irT_R > 95)
{

	if (irT_R > 95)
	{
		tR = 1;
		if (irT_R > 105)
		{
			tR = tR + 1;
			if (irT_R > 115)
			{
				tR = tR + 1;
			}	
		}
	}
}	
else 
	{
		tR = 0;
	}	

t_diff = tL - tR;

	return t_diff;
	
}


int Accelerator(int speed)
{
	
	int L_Speed, R_Speed;
	 
	
	
	if (speed <= 0)
	{
		if (speed <= -5)
		{
			L_Speed = -99;
			R_Speed = 4;
		}
		else if (speed <= -4)
		{
			L_Speed = -90;
			R_Speed = 14;
		}
		else if (speed <= -3)	
		{
			L_Speed = -80;
			R_Speed = 24;
			//perfect
		}
		else if (speed <= -2)
		{
			L_Speed = -70;
			R_Speed = 34;
			//near perfect
		}
		else if (speed <= -1)
		{
			L_Speed = -62;
			R_Speed = 44;
		}
		else if (speed == 0)
		{
			L_Speed = 0;
			R_Speed = 0;
		}
	}
	else if (speed > 0)
	{
		if (speed >= 5)
		{
			L_Speed = -6;
			R_Speed = 99;
		}
		else if (speed >= 4)
		{
			L_Speed = -11;
			R_Speed = 94;
		}
		else if (speed >= 3)
		{
			L_Speed = -19;
			R_Speed = 84;
		}
		else if (speed >= 2)
		{
			L_Speed = -29;
			R_Speed = 74;
		}
		else if (speed >= 1)
		{
			L_Speed = -41;
			R_Speed = 64;
		}
		
	}
	else
	{
			L_Speed = 0;
			R_Speed = 0;
	}		

	
	
	if (speed < current_speed)
	{
		BrakeLight;
	}
	else 
		NoSignal;
		
	motorp(0, L_Speed);
   	motorp(1, R_Speed);

	return current_speed;
}


void turn_toward_ir()
{

	int temp_L;
	int temp_R;
	
	temp_L = analog(3);
	temp_R = analog(2);

	if (temp_L > temp_R)
		{
			turn(-3);
		}
	else if (temp_R > temp_L)
		{
			turn(3);
		}
	
}

int CDS_procT()
{

	int C_left, C_right, C_nose, XYZ;
		
	C_left 	= analog(5);
	C_right = analog(6);
	C_nose	= analog(1);
	
	XYZ = 0;
	
	if ((C_left < CL_calib) && (C_right < CR_calib) && (C_nose < CN_calib))
	{
		XYZ = 0;
	}
	else if (C_left > CL_calib)
	{
		XYZ = -2;
		last_seen = -3;
	}
	else if (C_right > CR_calib)
	{
		XYZ = 2;
		last_seen = 3;
	}
	else if (C_nose > CN_calib)
	{
		XYZ = 0;
	}
	else if ((C_left < CL_calib) && (C_right < CR_calib))
	{
		XYZ = 0;
	}
	
	return XYZ;
}

void Random_backup()
{
	int q;
	unsigned rand;
	
	rand = TCNT;
	
	if (rand & 0x0001)
	{
		turn(3);
	}
	else 
	{
		turn(-3);
	}
	
	
	q=(rand % 4096);
	printf("%d", q);
	if (q<2500) wait(i);
	else wait(2500);
}